-- $Name: A Dragon and the Tower$
-- $Version: 1.1.0$
-- $Author: Khaelenmore Thaal & Sabra Naikomy$
-- $Info: A tiny illustrated text adventure$

if instead.tiny then
  require "lib/tiny/stubs"
end;
require "lib/link"
require "noinv"
require 'snd'
require 'keys'
require "timer"
require "theme"
if not instead.tiny then
  require "lib/decor"
end;
require "prefs"
if not instead.tiny then
  require "lib/dire/theme"
end;
require 'lib/utils'
require "lib/locale"
if not instead.tiny then
  require "lib/fading"
  require "lib/smoke"
end

fmt.quotes = false;
fmt.filter = function(r)
    local utf8 = (std.ref 'game'.codepage == 'UTF-8' or std.ref 'game'.codepage == 'utf-8')
    
    if utf8 then
        r = r:gsub('"([^"]*)"', translate_now("common_locale", 'quotes'));
    end;
    
    r = r:gsub('<tab>', fmt.nb('    '));
    
    return r;
end;

game.act = translate('main_locale', 'empty_act');
game.use = function(s)
  return translate_now('main_locale', random_phrase({
    'empty_use_1',
    'empty_use_2',
    'empty_use_3',
    'empty_use_4',
    'empty_use_5'
  }));
end;
game.inv = translate("main_locale", 'empty_inv');

instead.notitle = function()
  return here().notitle;
end;

function get_graphic_scale()
  local rval = prefs.graphic_scale;
  
  if not rval then
    local w, h = diretheme.get_screen();
    local wsrec
    if diretheme.get_inches() < 7 then
      -- On a screen smaller than 7 inch
      wsrec = math.floor(w / 640);
      rval = math.floor(h / 480);
    else
      wsrec = math.floor(w / 800);
      rval = math.floor(h / 540);
    end;
    
    if wsrec < rval then
      rval = wsrec;
    end;
    
    if rval < 1 then
      rval = 1;
    end;
  end;
  
  return rval;
end;

function get_cursor_scale()
  local rval = prefs.cursor_scale;
  
  if not rval then
    rval = get_graphic_scale();
  end;
  
  return rval;
end;

function get_font_scale()
  local rval = prefs.font_scale;
  if not rval then
    rval = get_graphic_scale() * 100;
    
    if diretheme.get_inches() < 7 then
      rval = rval + math.floor(rval / 2);
    end;
  end;
  
  return rval;
end;

function get_inventory_font_scale()
  local rval = prefs.inventory_scale;
  if not rval then
    rval = get_font_scale();
  end;
  return rval;
end;

instead.nosave = function()
  return here().nosave
end

instead.noautosave = function()
  return here().noautosave
end

std.strip_call = false;
std.phrase_show = false;

declare {
  current_theme = false,
  sound_names = {
    ['bubble'] = 'sound/bubble.ogg';
    ['potion'] = 'sound/potion.ogg';
    ['error_beep'] = 'sound/error_beep.ogg';
    ['long_beep'] = 'sound/long_beep.ogg';
    ['short_beep'] = 'sound/short_beep.ogg';
    ['note_c'] = 'sound/c.ogg';
    ['note_d'] = 'sound/d.ogg';
    ['note_e'] = 'sound/e.ogg';
    ['note_f'] = 'sound/f.ogg';
    ['note_g'] = 'sound/g.ogg';
    ['note_a'] = 'sound/a.ogg';
    ['note_b'] = 'sound/b.ogg';
    ['wind_gust_1'] = 'sound/wind_blow_1.ogg';
    ['wind_gust_2'] = 'sound/wind_blow_2.ogg';
    ['wind_gust_3'] = 'sound/wind_blow_3.ogg';
    ['open_door'] = 'sound/open_door.ogg';
    ['pc_startup'] = 'sound/pc_startup.ogg';
    ['keyboard1'] = 'sound/keyboard1.ogg';
    ['keyboard2'] = 'sound/keyboard2.ogg';
    ['keyboard3'] = 'sound/keyboard3.ogg';
    ['keyboard4'] = 'sound/keyboard4.ogg';
    ['keyboard5'] = 'sound/keyboard5.ogg';
    ['irradiated'] = 'sound/irradiated.ogg';
  };
  sounds = {},
  gusts = {
    'wind_gust_1';
    'wind_gust_2';
    'wind_gust_3';
  },
  menu_sprites = {
  },
  last_location = false
}

global {
  in_game = false,
}

function load_sounds(...)
  local args = {...}
  for i = 1, #args do
    local s = args[i];
    if not sounds[s] then
      sounds[s] = snd.new(sound_names[s]);
    end;
  end;
end;

if instead.tiny then
  function DS () end;
else
  function DS (data)
    local scale = diretheme.get_scale();
    data[3] = get_sprite(data[3], 'fuchsia');
    data['background'] = true;
    data['x'] = data['x'] * scale;
    data['y'] = data['y'] * scale;
    data['w'] = data['w'] * scale;
    data['h'] = data['h'] * scale;
    if data[2] == 'raw' then
      data['xc'] = 0;
      data['yc'] = 0;
    end;
    D(data);
  end;
end;

include "game_theme"
include "tutorial"
include "computer_parts"
include "solarscale"
include "tower"
include "forest"
include "ending"

if instead.tiny then
  include "tiny_menu"
else
  include "menu"
end;


game.ondecor = function(s, name, press, x, y, btn)
  if press then
    if name == 'settings_open' then
      in_game = true;
      walkin('intro_settings');
      return true;
    end;
  end;
  return false;
end;

game.afterwalk = function()
  if last_location then
    dprint(last_location, '->', here().nam);
    
    if _(last_location).themeDispose then
      _(last_location):themeDispose();
    end;
  end;
  
  local theme = std.call(here(), 'theme');
  
  if theme == nil then
    error("Theme is unset here: " .. tostring(here().nam));
  end;
  
  if theme ~= nil and theme ~= current_theme then
    dprint('Switch theme', current_theme, '->', theme);
    set_theme(theme);
  end;
  
  if here().themeInit then
    here():themeInit();
  end;
  
  last_location = here().nam;
end;

room {
  nosave = true;
  noautosave = true;
  nofollow = true;
  theme = 'black';
  noinv = true;
  nam = 'resolution_warning';
  title = translate("menu_locale", 'warning');
  decor = translate("menu_locale", 'resolution_warning');
  onenter = stop_menu;
}: with {
  obj {
    nam = '#sysset';
    dsc = function(s) return '{' .. translate_now("menu_locale", 'system_settings') .. '}'; end;
    act = function(s)
      instead.menu();
    end;
  }
}

room {
  nosave = true;
  noautosave = true;
  nofollow = true;
  theme = 'black';
  noinv = true;
  nam = 'theme_warning';
  title = translate("menu_locale", 'warning');
  decor = translate("menu_locale", 'theme_warning');
  onenter = stop_menu;
}: with {
  obj {
    nam = '#sysset';
    dsc = function(s) return '{' .. translate_now("menu_locale", 'system_settings') .. '}'; end;
    act = function(s)
      instead.menu();
    end;
  }
}

function start(load)
  dprint('start', load);
  last_location = here().nam;
  local w, h = diretheme.get_screen();
  local name = theme.name();
  if instead.tiny then
    if not load then
      take('tiny_ingame_settings');
    end;
  else
    if here() ~= _'theme_warning' and here() ~= _'resolution_warning' then
      if name ~= '.' then
        walkin('theme_warning');
        
        game:reaction(false);
        local r = game:display(true);
        game:lastdisp(r);
      end;
      
      if w < 640 or h < 480 then
        walkin('resolution_warning');
        
        game:reaction(false);
        local r = game:display(true);
        game:lastdisp(r);
      end;
    end;
  end;
  
  set_theme(std.call(here(), 'theme'));
  
  diretheme.set_theme();
  
  if load then
    if here().themeInit then
      here():themeInit();
    end;

    if here().onboot then
      here():onboot();
    end;
  else
    local words = {
      "biography",
      "algorithm",
      "chemistry",
      "discovery",
      "exploring",
      "machinery",
      "porcelain",
      "something",
      "wonderful"
    };    
    
    local maps = {
      {6, 1, 4, 8, 2, 3, 7, 5, 9};
      {1, 3, 8, 6, 2, 9, 4, 5, 7};
      {1, 3, 2, 5, 4, 8, 6, 7, 9};
      {4, 1, 7, 2, 5, 8, 3, 6, 9};
      {1, 9, 7, 4, 8, 5, 3, 6, 2};
      {2, 3, 7, 4, 5, 1, 6, 8, 9};
      {7, 4, 5, 8, 6, 9, 2, 1, 3};
      {4, 9, 6, 7, 3, 8, 2, 1, 5};
      {4, 5, 7, 9, 3, 2, 6, 8, 1};
    };
    
    local wordIndex = rnd(#words);
    
    library_word = words[wordIndex];
    library_book_map = maps[wordIndex];
    
    local keys = {1,2,3,4};
    for i = 0, 3 do
      local index = rnd(#keys);
      _('tower_key_' .. tostring(i)).value = keys[index];
      keys[index] = keys[#keys];
      keys[#keys] = nil;
    end;
    
    local keyholes = {125, 25, 5, 1};
    for i = 1, 4 do
      local index = rnd(#keyholes);
      _('keyhole_' .. tostring(i)).value = keyholes[index];
      keyholes[index] = keyholes[#keyholes];
      keyholes[#keyholes] = nil;
    end;
    
    keyholes = {125, 25, 5, 1};
    local value = 0;
    for i = 1, 4 do 
      local index = rnd(#keyholes);
      value = value + keyholes[index] * i;
      keyholes[index] = keyholes[#keyholes];
      keyholes[#keyholes] = nil;
    end;
    level_1_target = tostring(value);
    
    local catalysts = {
      'red_catalyst';
      'green_catalyst';
      'blue_catalyst';
      'turquoise_catalyst';
      'yellow_catalyst';
      'purple_catalyst';
    };
    
    local index = rnd(#catalysts);
    opening_catalyst = catalysts[index];
    catalysts[index] = catalysts[#catalysts];
    catalysts[#catalysts] = nil;
    
    index = rnd(#catalysts);
    light_catalyst = catalysts[index];
    catalysts[index] = catalysts[#catalysts];
    catalysts[#catalysts] = nil;
    
    index = rnd(#catalysts);
    enrichment_catalyst = catalysts[index];
    
    password1 = make_password();
    password2 = make_password();
    
    local computer_parts = {
      'cvx-rst-i3sb',
      'mcpu70-cvx-7mhz',
      'rst-m16-16mb',
      'tva-rx1',
      'fna-100-10mbs_1',
      'kbc-s'
    };
    
    local bpi = rnd(#computer_parts);
    _(computer_parts[bpi]).status = false;
    computer_parts[bpi] = computer_parts[#computer_parts];
    computer_parts[#computer_parts] = nil;
 
    bpi = rnd(#computer_parts);
    _(computer_parts[bpi]).status = false;
    
    -- Debug
  end;
end;

function init()
  if not prefs.graphic_scale then
    prefs.graphic_scale = false;
  end;
  
  if not prefs.cursor_scale then
    prefs.cursor_scale = false;
  end;
  
  if not prefs.text_input then
    prefs.text_input = true;
  end;
  
  if not prefs.completed then
    prefs.completed = false;
  end;
  
  if not prefs.font_scale then
    prefs.font_scale = false;
  end;
  
  if not prefs.inventory_scale then
    prefs.inventory_scale = false;
  end;
  
  if not prefs.text_alignment then
    prefs.text_alignment = 1;
  end;
  
  if not prefs.text_padding then
    prefs.text_padding = 1;
  end;
  
  if not prefs.text_interval then
    prefs.text_interval = 1;
  end;
  
  if not prefs.text_width then
    prefs.text_width = 100;
  end;
  
  if not instead.tiny and prefs.simplified_animation == nil then
    -- Tiny performance test
    local a = sprite.new(640, 480);
    local b = sprite.new(640, 480);
    local t = instead.ticks();
    b:copy(a, 0, 0);
    t = instead.ticks() - t;
    dprint(t);
    prefs.simplified_animation = t > 10;
  end;
end;
